'$INCLUDE: 'EVENTS.BI'
'$DYNAMIC

TYPE RegType
     AX    AS INTEGER
     BX    AS INTEGER
     CX    AS INTEGER
     DX    AS INTEGER
     BP    AS INTEGER
     SI    AS INTEGER
     DI    AS INTEGER
     FLAGS AS INTEGER
END TYPE

TYPE RegTypeX
     AX    AS INTEGER
     BX    AS INTEGER
     CX    AS INTEGER
     DX    AS INTEGER
     BP    AS INTEGER
     SI    AS INTEGER
     DI    AS INTEGER
     FLAGS AS INTEGER
     DS    AS INTEGER
     ES    AS INTEGER
END TYPE

DECLARE SUB INTERRUPT ( InterruptNumber AS INTEGER, InRegister AS RegType, OutRegister AS RegType )
DECLARE SUB INTERRUPTX ( InterruptNumber AS INTEGER, InRegister AS RegTypeX, OutRegister AS RegTypeX )

DECLARE FUNCTION MouseDriverPresent% ( Buttons% )
REM ***********************************************************************
REM * This function tests for the presence of a microsoft compatible      *
REM * mouse driver.  If one is detected, this routine sets all the        *
REM * internal variables to their defaults, the number of buttons on the  *
REM * mouse is returned in Buttons% and the function returns TRUE%.  If   *
REM * no mouse driver is detected FALSE% is returned.  This routine is    *
REM * called by OpenEventSystem, so it does not need to be called if that *
REM * routine has been used.                                              *
REM ***********************************************************************

DECLARE SUB GetMouseStatus ( m AS MouseDetails )
REM ***********************************************************************
REM * This routine takes a snapshot of the condition of the mouse at the  *
REM * time that this routine is called.  The information that is returned *
REM * relates to the press state of the buttons on the mouse and the      *
REM * coordinates of the position of mouse pointer on screen.             *
REM ***********************************************************************

DECLARE FUNCTION KeyPressWaiting% ()
REM ***********************************************************************
REM * This function tests whether or not there is an unprocessed          *
REM * keystroke present in the keyboard buffer.  It returns TRUE% if this *
REM * is so and FALSE% otherwise.                                         *
REM ***********************************************************************

DECLARE FUNCTION GetKeyScan% ()
REM ***********************************************************************
REM * This function reads a keystroke from the keyboard and returns the   *
REM * full scancode of the key pressed.  This function is best used in    *
REM * conjunction with the preceding one in the following manner.         *
REM *                                                                     *
REM *             IF KeyPressWaiting% THEN                                *
REM *                 MyKeyCode% = GetKeyScan%                            *
REM *                 ....                                                *
REM *             END IF                                                  *
REM *                                                                     *
REM ***********************************************************************

DECLARE FUNCTION PeekTicks% ()
REM ***********************************************************************
REM * This routine (written in assembly for speed) returns the            *
REM * low-order word of the systen timer-tick counter.                    *
REM ***********************************************************************

REM ***********************************************************************
REM * The following are what I consider to be K.I.S.S.ware private        *
REM * routines for this small library and their individual functions are  *
REM * hopefully obvious from their names.                                 *
REM ***********************************************************************

DECLARE FUNCTION CompareMice% ( A AS MouseDetails, B AS MouseDetails, WhatToTest% )
DECLARE SUB TimerOnly ( MyEvent AS EventResult )
DECLARE SUB KeyboardOnly ( MyEvent AS EventResult )
DECLARE SUB MouseOnly ( MyEvent AS EventResult )
DECLARE SUB KeyboardTimer ( MyEvent AS EventResult )
DECLARE SUB MouseTimer ( MyEvent AS EventResult )
DECLARE SUB MouseKeyboard ( MyEvent AS EventResult )
DECLARE SUB GetAny ( MyEvent AS EventResult )

CONST ZeroFlag% = 64

DIM SHARED GotMouse AS INTEGER

SUB OpenEventSystem
    IF NOT MouseDriverPresent%(MouseButtons%) THEN
        GotMouse = FALSE%
    ELSE
        GotMouse = TRUE%
    END IF
END SUB

FUNCTION WeHaveAMouse%
    WeHaveAMouse% = GotMouse
END FUNCTION

SUB GetEvent ( MyEvent AS EventResult )
    IF (((MyEvent.EventType AND TimerEvent%) <> 0) AND MyEvent.TicksLeft < 1) THEN
        CLS
        PRINT "Error in GetEvent - "
        PRINT "TimerEvent Selected and no time set!"
        PRINT "Aborting NOW!"
        STOP
    END IF
    IF ((MyEvent.EventType AND MouseEvent%) <> 0) AND NOT GotMouse THEN
        MyEvent.EventType = (MyEvent.EventType AND 7)
    END IF
    IF (((MyEvent.EventType AND MouseEvent%) <> 0) AND (MyEvent.EventType% AND AnyMouseEvent%) = 0) THEN
        SCREEN 0
        CLS
        PRINT "Error in GetEvent - "
        PRINT "MouseEvent Selected and no type of mouse event specified!"
        PRINT "Aborting NOW!"
        STOP
    END IF
    IF (MyEvent.EventType AND AnyEvent%) = AnyEvent% THEN
        GetAny MyEvent
    ELSEIF (MyEvent.EventType AND MouseEvent%) <> 0 THEN
        IF (MyEvent.EventType AND TimerEvent%) <> 0 THEN
            MouseTimer MyEvent
        ELSEIF (MyEvent.EventType AND KeyEvent%) <> 0 THEN
            MouseKeyboard MyEvent
        ELSE
            MouseOnly MyEvent
        END IF
    ELSEIF (MyEvent.EventType AND KeyEvent%) <> 0 THEN
        IF (MyEvent.EventType AND TimerEvent%) <> 0 THEN
            KeyboardTimer MyEvent
        ELSE
            KeyBoardOnly MyEvent
        END IF
    ELSE
        TimerOnly MyEvent
    END IF
END SUB

FUNCTION CompareMice% ( A AS MouseDetails, B AS MouseDetails, WhatToTest% )
    TestThis% = WhatToTest% AND &HF0
    TheResult% = 0
    IF (TestThis% AND LeftClick%) <> 0 THEN
        TheResult% = TheResult% OR (A.LeftPressed XOR B.LeftPressed)
    END IF
    IF (TestThis% AND RightClick%) <> 0 THEN
        TheResult% = TheResult% OR (A.RightPressed XOR B.RightPressed)
    END IF
    IF (TestThis% AND CenterClick%) <> 0 THEN
        TheResult% = TheResult% OR (A.CenterPressed XOR B.CenterPressed)
    END IF
    IF (TestThis% AND MousePosition%) <> 0 THEN
        TheResult% = TheResult% OR (A.XPosition XOR B.XPosition) OR (A.YPosition XOR B.YPosition)
    END IF
    CompareMice% = (TheResult% <> 0)
END FUNCTION

SUB TimerOnly ( MyEvent AS EventResult )
    EndTime& = CLNG(PeekTicks%) + CLNG(MyEvent.TicksLeft)
    DO
        CurrentTime& = CLNG(PeekTicks%)
    LOOP UNTIL CurrentTime& >= EndTime&
    MyEvent.TicksLeft = 0
END SUB

SUB KeyboardOnly ( MyEvent AS EventResult )
    DO
    LOOP UNTIL KeyPressWaiting%
    MyEvent.ScanCode = GetKeyScan%
END SUB

SUB MouseOnly ( MyEvent AS EventResult )
    DIM OldMouse AS MouseDetails, NewMouse AS MouseDetails
    GetMouseStatus OldMouse
    DO
        GetMouseStatus NewMouse
        Changed% = CompareMice(OldMouse, NewMouse, MyEvent.EventType)
    LOOP UNTIL Changed%
    MyEvent.Mouse = NewMouse
    MyEvent.EventType = MouseEvent%
END SUB

SUB KeyboardTimer ( MyEvent AS EventResult )
    EndTime& = CLNG(PeekTicks%) + CLNG(MyEvent.TicksLeft)
    DO
        CurrentTime& = CLNG(PeekTicks%)
    LOOP UNTIL (CurrentTime& >= EndTime&) OR KeyPressWaiting%
    IF CurrentTime& < EndTime& THEN
        MyEvent.ScanCode = GetKeyScan%
        MyEvent.TicksLeft = CINT(EndTime& - CurrentTime&)
        MyEvent.EventType = KeyEvent%
    ELSE
        MyEvent.TicksLeft = 0
        MyEvent.EventType = TimerEvent%
    END IF
END SUB

SUB MouseTimer ( MyEvent AS EventResult )
    DIM OldMouse AS MouseDetails, NewMouse AS MouseDetails
    GetMouseStatus OldMouse
    EndTime& = CLNG(PeekTicks%) + CLNG(MyEvent.TicksLeft)
    DO
        GetMouseStatus NewMouse
        Changed% = CompareMice(OldMouse, NewMouse, MyEvent.EventType)
        CurrentTime& = CLNG(PeekTicks%)
    LOOP UNTIL (CurrentTime& >= EndTime&) OR Changed%
    IF Changed% THEN
        MyEvent.Mouse = NewMouse
        MyEvent.TicksLeft = CINT(EndTime& - CurrentTime&)
        MyEvent.EventType = MouseEvent%
    ELSE
        MyEvent.TicksLeft = 0
        MyEvent.EventType = TimerEvent%
    END IF
END SUB

SUB MouseKeyboard ( MyEvent AS EventResult )
    DIM OldMouse AS MouseDetails, NewMouse AS MouseDetails
    GetMouseStatus OldMouse
    DO
        GetMouseStatus NewMouse
        Changed% = CompareMice(OldMouse, NewMouse, MyEvent.EventType)
    LOOP UNTIL KeyPressWaiting% OR Changed%
    IF Changed% THEN
        MyEvent.Mouse = NewMouse
        MyEvent.EventType = MouseEvent%
    ELSE
        MyEvent.ScanCode = GetKeyScan%
        MyEvent.EventType = KeyEvent%
    END IF
END SUB

SUB GetAny ( MyEvent AS EventResult )
    DIM OldMouse AS MouseDetails, NewMouse AS MouseDetails
    GetMouseStatus OldMouse
    EndTime& = CLNG(PeekTicks%) + CLNG(MyEvent.TicksLeft)
    DO
        GetMouseStatus NewMouse
        Changed% = CompareMice(OldMouse, NewMouse, MyEvent.EventType)
        GotKey% = KeyPressWaiting%
        CurrentTime& = CLNG(PeekTicks%)
    LOOP UNTIL GotKey% OR Changed% OR (CurrentTime& >= EndTime&)
    IF Changed% THEN
        MyEvent.Mouse = NewMouse
        MyEvent.TicksLeft = CINT(EndTime& - CurrentTime&)
        MyEvent.EventType = MouseEvent%
    ELSEIF GotKey% THEN
        MyEvent.ScanCode = GetKeyScan%
        MyEvent.TicksLeft = CINT(EndTime& - CurrentTime&)
        MyEvent.EventType = KeyEvent%
    ELSE
        MyEvent.TicksLeft = 0
        MyEvent.EventType = TimerEvent%
    END IF
END SUB

FUNCTION MouseDriverPresent% ( Buttons% )
    DIM InReg AS RegType, OutReg AS RegType
    InReg.AX = 0
    Interrupt &H33, InReg, OutReg
    IF OutReg.AX = &HFFFF THEN
        Buttons% = OutReg.BX
        MouseDriverPresent% = TRUE%
    ELSE
        MouseDriverPresent% = FALSE%
    END IF
END FUNCTION

SUB ShowMousePointer
    DIM InReg AS RegType, OutReg AS RegType
    InReg.AX = 1
    Interrupt &H33, InReg, OutReg
END SUB

SUB HideMousePointer
    DIM InReg AS RegType, OutReg AS RegType
    InReg.AX = 2
    Interrupt &H33, InReg, OutReg
END SUB

SUB GetMouseStatus ( m AS MouseDetails )
    DIM InReg AS RegType, OutReg AS RegType
    InReg.AX = 3
    Interrupt &H33, InReg, OutReg
    m.XPosition = OutReg.CX
    m.YPosition = OutReg.DX
    m.LeftPressed = (LeftButton% AND OutReg.BX)
    m.RightPressed = (RightButton% AND OutReg.BX)
    m.CenterPressed = (CenterButton% AND OutReg.BX)
END SUB

SUB SetPointerPosition ( X%, Y% )
    DIM InReg AS RegType, OutReg AS RegType
    IF X% >= 0 AND Y% >= 0 THEN
        InReg.AX = 4
        InReg.CX = X%
        InReg.DX = Y%
        Interrupt &H33, InReg, OutReg
    END IF
END SUB

SUB SetGraphicsPointerShape ( HotSpotLeftOffset%, HotSpotTopOffset%, PointerShape%() )
    DIM InReg AS RegTypeX, OutReg AS RegTypeX
    QIndex% = LBOUND(PointerShape%)
    IF HotSpotLeftOffset% < -16 OR HotSpotTopOffset% < -16 THEN
        EXIT SUB
    END IF
    IF HotSpotLeftOffset% > 16 OR HotSpotTopOffset% > 16 THEN
        EXIT SUB
    END IF
    IF LBOUND(PointerShape%) = 0 AND UBOUND(PointerShape%) = 31 THEN
        ShapeSeg% = VARSEG(PointerShape%(QIndex%))
        ShapeOff% = VARPTR(PointerShape%(QIndex%))
        InReg.AX = 9
        InReg.BX = HotSpotLeftOffset%
        InReg.CX = HotSpotTopOffset%
        InReg.ES = ShapeSeg%
        InReg.DX = ShapeOff%
        InterruptX &H33, InReg, OutReg
    END IF
END SUB

FUNCTION GetKeyScan%
    DIM InReg AS RegType, OutReg AS RegType
    InReg.AX = &H1000
    Interrupt &H16, InReg, OutReg
    GetKeyScan% = OutReg.AX
END FUNCTION

FUNCTION KeyPressWaiting%
    DIM InReg AS RegType, OutReg AS RegType
    InReg.AX = &H1100
    Interrupt &H16, InReg, OutReg
    KeyPressWaiting% = ((OutReg.FLAGS AND ZeroFlag%) = 0)
END FUNCTION


