'$DYNAMIC

TYPE MouseDetails
    LeftPressed   AS INTEGER
    RightPressed  AS INTEGER
    CenterPressed AS INTEGER
    XPosition     AS INTEGER
    YPosition     AS INTEGER
END TYPE

REM **********************************************************************
REM * This type encapsulates all of the information necessary for mouse  *
REM * usage so as to minimise the number of parameters that need to be   *
REM * passed between routines.  The first three parts of this type are   *
REM * boolean (i.e. TRUE% or FALSE%) in nature.  Whereas the remaining   *
REM * two give the position of the pointer's hot-spot (in pixel          *
REM * coordinates) on the screen.                                        *
REM **********************************************************************

TYPE EventResult
    EventType AS INTEGER
    ScanCode  AS INTEGER
    TicksLeft AS INTEGER
    Mouse     AS MouseDetails
END TYPE

REM **********************************************************************
REM * This type is used both to pass information to and get information  *
REM * from the GetEvent routine.  You only ever need a single variable   *
REM * of this type in a program.                                         *
REM **********************************************************************

CONST FALSE% = 0, TRUE% = NOT FALSE%
CONST UpArrow% = &H48E0, DownArrow% = &H50E0, ReturnKey% = &H1C0D
CONST EscapeKey% = &H11B, LeftArrow% = &H4BE0, RightArrow% = &H4DE0
CONST Home% = &H47E0, EndKey% = &H4FE0, PageUp% = &H49E0, PageDown% = &H51E0
CONST Insert% = &H52E0, Delete% = &H53E0, BackSpace% = &HE08
CONST MouseResult% = 0, LeftButton% = 1, RightButton% = 2, CenterButton% = 4
CONST MouseEvent% = 8, KeyEvent% = 4, TimerEvent% = 2, AnyEvent% = 15
CONST LeftClick% = &H80, RightClick% = &H40, CenterClick% = &H20
CONST MousePosition% = &H10, AnyMouseEvent% = &HF0

DECLARE SUB OpenEventSystem ()
REM **********************************************************************
REM * It is necessary to call this routine once (and once only) before   *
REM * using any other routines in this mini-library.                     *
REM **********************************************************************

DECLARE SUB GetEvent ( MyEvent AS EventResult )
REM **********************************************************************
REM * This routine is the reason for the construction of this            *
REM * mini-library.  It takes a single parameter and returns it altered. *
REM * The purpose of this routine is to simplify input where both a      *
REM * mouse and a keyboard have to be simultaneously monitored.  It      *
REM * also incorperates a timing mechanism based upon the system         *
REM * "ticker".                                                          *
REM **********************************************************************

DECLARE FUNCTION WeHaveAMouse%()
REM **********************************************************************
REM * Once OpenEventSystem has been called, this routine will return     *
REM * TRUE% if a microsoft compatible mouse is present and FALSE%        *
REM * otherwise.  Calling this routine before OpenEventSystem is NOT     *
REM * advisable and the value returned will be totally meaningless.      *
REM **********************************************************************

DECLARE SUB ShowMousePointer ()
DECLARE SUB HideMousePointer ()
REM **********************************************************************
REM * These 2 routines show and hide the mouse pointer, respectively.    *
REM **********************************************************************


DECLARE SUB SetPointerPosition ( X%, Y% )
REM **********************************************************************
REM * This routine resets the mouse pointer so that its hot-spot is      *
REM * moved to the absolute position specified by X%, Y%.                *
REM **********************************************************************

DECLARE SUB SetGraphicsPointerShape ( HotSpotLeftOffset%, HotSpotTopOffset%, PointerShape%() )
REM **********************************************************************
REM * This routine is used to alter the shape of the mouse pointer on    *
REM * the graphics screen.                                               *
REM **********************************************************************


